<?php
namespace DistributeHelper;
error_reporting(E_ALL & ~E_NOTICE);
include_once "Utility.php";

class WscriptShell
{
	public static $shell = null;
	public function __construct()
	{
		try{
			if(self::$shell == null)
				self::$shell = new \com("WScript.Shell");
		}catch (\Exception $e) {
			 logError('failed to creeate WScript.Shell',$e->getCode(),$e->getMessage());
		}
	}
	public static function pathJoin($base, $path) {
		$base = str_replace('\\','/',$base);
		$path = str_replace('\\','/',$path);
		return rtrim( $base, '/' ) . '/' . ltrim( $path, '/' );
	}
	/* style
	0 SW_HIDE
	1 SW_SHOWNORMAL
	2 SW_SHOWMINIMIZED
	3 SW_SHOWMAXIMIZED
	4 SW_SHOWNOACTIVATE
	5 SW_SHOW
	6 SW_MINIMIZE
	7 SW_SHOWMINNOACTIVE
	8 SW_SHOWNA
	9 SW_RESTORE
	*/
	public static function Run($cmd,$style,$wait)
	{
		$shell = self::$shell;
		if(! $shell){
			logError('failed to Run. shell is null.');
			return false;
		}
		try{
			$code = $shell->Run($cmd,$style,$wait);
			if($code != 0){
				logError('failed to Run',$cmd, 'code=',$code);
			}
			return $code;
		} catch (\Exception $e) {
			logError('failed to Run',$cmd, $e->getCode(),$e->getMessage());
			return false;
		}
	}
	public static function RunOutput($cmd,$out_file) //运行并获取输出文本数组
	{
		$shell = self::$shell;
		if(! $shell){
			logError('failed to Run. shell is null.');
			return false;
		}
		if( file_exists($out_file) ){
			unlink($out_file);
		}
		$cmd = "cmd.exe /C {$cmd} > {$out_file}";
		$ret = self::Run($cmd,0,1);
		if( $ret === false){
			logError('failed to RunOutput',$cmd);
			return false;
		}
		if( $ret !== 0){
			logError('failed to RunOutput',$cmd, 'code',$ret);
			return false;
		}
		if( !file_exists($out_file) ){
			logError('failed to find out file',$out_file);
			return false;
		}
		return $ret;
	}
	public static function RegRead($key)
	{
		$shell = self::$shell;
		if(! $shell){
			logError('failed to RegRead. shell is null.');
			return false;
		}
		try{
			return $shell->RegRead($key);
		} catch (\Exception $e) {
			logError('failed to RegRead',$e->getCode(),$e->getMessage());
			return false;
		}
	}
	public static function RegWrite($key,$value,$type)
	{
		$shell = self::$shell;
		if(! $shell){
			logError('failed to RegWrite. shell is null.');
			return false;
		}
		try{
			$shell->RegWrite($key,$value,$type);
			return true;
		} catch (\Exception $e) {
			logError('failed to RegWrite',$e->getCode(),$e->getMessage());
			return false;
		}
	}
	public static function RegDelete($key)
	{
		$shell = self::$shell;
		if(! $shell){
			logError('failed to RegDelete. shell is null.');
			return false;
		}
		//check if key exists
		try{
			$shell->RegRead($key);
		} catch (\Exception $e) {
			logInfo('reg key not exists. skip delete',$key);
			return true;
		}
		//delete key
		try{
			$shell->RegDelete($key);
			return true;
		} catch (\Exception $e) {
			logError('failed to RegDelete',$e->getCode(),$e->getMessage());
			return false;
		}
	}
	public static function createShortcut($shortcutPath,$targetPath,$description = '')
	{
		$shell = self::$shell;
		if(! $shell){
			logError('failed to createShortcut. shell is null.');
			return false;
		}
		try{
			$link = $shell->CreateShortcut($shortcutPath);
			$link->TargetPath = $targetPath;
			$link->WorkingDirectory = dirname($targetPath);
			$link->Description = $description;
			$link->Save();
			return true;
		} catch (\Exception $e) {
			 logError('failed to createShortcut',$e->getCode(),$e->getMessage());
			 return false;
		}
	}
	public static function getDesktopFolder()
	{
		$shell = self::$shell;
		if(! $shell){
			logError('failed to getDesktopFolder. shell is null.');
			return false;
		}
		try{
			return $shell->SpecialFolders("Desktop");
		} catch (\Exception $e) {
			 logError('failed to getDesktopFolder',$e->getCode(),$e->getMessage());
			 return false;
		}
	}
}
//windows进程操作
class WinProcess extends WscriptShell
{
	private static $tmp_dir;
	public function __construct($tmp_dir)
	{
		WscriptShell::__construct();
		self::$tmp_dir = $tmp_dir;
	}
	public static function find($name){
		$out_file = pathJoin(self::$tmp_dir,$name . '.txt');
		$cmd = "tasklist /FI \"IMAGENAME eq {$name}\" /FO list";
		if( self::RunOutput($cmd,$out_file) !== 0){
			logError('failed to RunOutput',$cmd);
			return false;
		}
		$pids = array();
		$fp = fopen($out_file,"r");
		if(!$fp){
			logError('failed to open',$out_file);
			return false;
		}
		while($line = fgets($fp) ){
			if( substr($line,0,4) == 'PID:'){
				$pid = trim(substr($line,4));
				array_push($pid);
			}
		}
		fclose($fp);
		return $pids;
	}
	
	public static function remove($name)
	{
		//关闭进程
		$cmd = "cmd.exe /C taskkill /F /IM {name}";
		if( self::Run($cmd,0,1) !== 0){
			logError('failed to Run',$cmd);
			return false;
		}
		
		//检测是否已经没有该任务进程
		$pids = self::find($name);
		if($pids === false){
			logError('failed to execute find',$name);
			return false;
		}
		if( count($pids) == 0){
			logInfo('no process exists',$name);
			return true;
		}
		else{
			logInfo('process still exists after stop',$name);
			return false;
		}
	}
};

//windows系统是否有管理员权限，写入一个注册表键值测试
class WinAdminPriv extends WscriptShell
{
	private static $KEY_PATH = "HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment\\DistHelper";
	public function __construct()
	{
		WscriptShell::__construct();
	}
	public static function check()
	{
		return self::RegWrite(self::$KEY_PATH,'AdminPriv','REG_EXPAND_SZ');
	}
};

//windows系统环境路径操作类
class WinEnvPath extends WscriptShell
{
	private static $KEY_PATH = "HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment\\Path";
	public function __construct()
	{
		WscriptShell::__construct();
	}
	public static function readPath()
	{
		return self::RegRead(self::$KEY_PATH);
	}
	public static function updatePath(array $removePatterns ,array $addPathes,array &$removePathes)
	{
		$envPath = self::RegRead(self::$KEY_PATH);
		if($envPath === false){
			return false;
		}
		$newPath = self::removePath($envPath,$removePatterns,$removePathes);
		$newPath = self::addPath($newPath,$addPathes);
		return self::RegWrite(self::$KEY_PATH,$newPath,'REG_EXPAND_SZ');
	}
	private static function isPattern(string $path,array $patterns)
	{
		foreach($patterns as $pattern){
			if(strpos($path,$pattern) !== false){
				return true;
			}
		}
		return false;
	}
	private static function removePath(string $envPath,array $patterns,array &$removePathes)
	{
		$array = explode(';',$envPath);
		$result = array();
		foreach($array as $path){
			if( self::isPattern($path,$patterns)){
				array_push($removePathes,$path);
			}
			else{
				array_push($result,$path);
			}
		}
		return implode(';',$result);
	}
	private static function addPath(string $envPath,array $addPathes)
	{
		$array = explode(';',$envPath);
		$result = array_merge($array,$addPathes);
		return implode(';',$result);
	}
};

//windows系统自启动操作类
class WinAutorun extends WscriptShell
{
	private static $KEY_PATH = "HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run\\";
	public function __construct()
	{
		WscriptShell::__construct();
	}
	public static function remove($name)
	{
		if( strlen($name) == 0){ 
			logError('failed to removeAutorun. name is empty.');
			return false;
		}
		$key = self::$KEY_PATH . $name;
		return self::RegDelete($key);
	}
	public static function register($name,$cmd)
	{
		if( strlen($name) == 0){
			logError('failed to registerAutorun. name is empty.');
			return false;
		}
		if( strlen($cmd) == 0){
			logError('failed to registerAutorun. cmd is empty.');
			return false;
		}
		$key = self::$KEY_PATH . $name;
		return self::RegWrite($key,$cmd,'REG_EXPAND_SZ');
	}
};

//windows 快捷键操作
class WinDesktopShortcut extends WscriptShell
{
	public function __construct()
	{
		WscriptShell::__construct();
	}
	public static function remove($shortcutName)
	{
		$desktopFolder = self::getDesktopFolder();
		if(! $desktopFolder){
			return false;
		}
		$shortcutPath = pathJoin($desktopFolder,$shortcutName);
		if( file_exists($shortcutPath) )
			return unlink($shortcutPath);
		else
			return true;
	}
	public static function register($shortcutName,$targetPath,$description)
	{
		$desktopFolder = self::getDesktopFolder();
		if(! $desktopFolder){
			return false;
		}
		$shortcutPath = pathJoin($desktopFolder,$shortcutName);
		return self::createShortcut($shortcutPath,$targetPath,$description);
	}
};

//windows文件操作
class WinFile extends WscriptShell
{
	public function __construct()
	{
		WscriptShell::__construct();
	}
	public static function setHidden($path)
	{
		$cmd = "cmd.exe /C attrib \"{$path}\" +s +h";
		return self::Run($cmd,0,1) === 0;
	}	
};
function setHiddenAttrib($path){
	$os = getOsName();
	if( $os == 'win'){
		$wf = new WinFile();
		return $wf->setHidden($path);
	}
	else{
		return true;
	}
}
//windows服务操作
class WinService extends WscriptShell
{
	private static $tmp_dir;
	public function __construct($tmp_dir)
	{
		WscriptShell::__construct();
		self::$tmp_dir = $tmp_dir;
	}
	public static function register($name,$path)
	{
		if(stripos($name,'mysql') !== false){
			return self::mysqldInstall($name,$path);
		}
		else{
			logError('failed to register service. not support name.',$name,$path);
			return false;
		}
	}
	public static function remove($name)
	{
		//先停止服务
		if(! self::scstop($name) ){
			logError('failed to stop service.',$name);
			return false;
		}
		//再移除服务
		if(! self::scdelete($name)){
			logError('failed to delete service.',$name);
			return false;
		}
		return true;
	}
	
	// false: failed
	// 0: exists
	// 1060: not exists
	// other int: other error
	public static function scexists($name)
	{
		$cmd = "sc query {$name}"; 
		return self::Run($cmd,0,1);
	}
	// false: failed
	// 1060: not exists
	// other int: other error
	// array: service info
	// *** not implemented ***
	public static function scquery($name)
	{
		$cmd = "sc query {$name}"; 
		return self::Run($cmd,0,1) === 0;
	}
	public static function scstart($name)
	{
		$cmd = "sc start {$name}";
		return self::Run($cmd,0,1) === 0;
	}
	public static function scdelete($name)
	{
		$cmd = "sc delete {$name}";
		$code = self::Run($cmd,0,1);
		if($code === false){
			return false;
		}
		else if($code === 0){
			logInfo('service deleted successfully.',$name);
			return true;
		}
		else if($code === 1060){
			logInfo('service not exists. no need to delete.',$name);
			return true;
		}
		else{
			logError('unknow error during delete service.',$name,$code);
			return false;
		}
	}
	// false: execute failed
	// true: stop succeeded ( service not exists, service already stopped )
	public static function scstop($name)
	{
		$cmd = "sc stop {$name}";
		$code = self::Run($cmd,0,1);
		if($code === false){
			return false;
		}
		else if($code === 0){
			logInfo('service stopped successfully.',$name);
			return true;
		}
		else if($code === 1060){
			logInfo('service not exists. no need to stop.',$name);
			return true;
		}
		else if($code === 1062){
			logInfo('service not running. no need to stop.',$name);
			return true;
		}else{
			logError('unknow error during stopping service.',$name,$code);
			return false;
		}
	}
	private static function mysqldInstall($name,$path)
	{
		$cmd = "{$path} --install {$name}";
		$code = self::Run($cmd,0,1);
		if($code === 0){
			logError('succeeded to install mysql service.',$name,$path);
			return true;
		}
		else{
			logError('failed to install mysql service.',$code,$name,$path);
			return false;
		}
	}
	private static function mysqldRemove($path)
	{
		$cmd = "{$path} --remove";
		$code =  self::Run($cmd,0,1);
		if($code === 0){
			logError('failed to remove mysql service.',$name,$path);
			return true;
		}
		else{
			return false;
		}
	}
};
